#!/usr/bin/env python
# coding=utf-8
"""
@author: Jiawei Wu
@create time: 2020-03-18 11:52
@edit time: 2020-04-07 19:41
@FilePath: /vvlab/__version__.py
"""



VERSION = (0, 1, 8)

__version__ = '.'.join(map(str, VERSION))
