#!/usr/bin/env python
# coding=utf-8
"""
@author: Jiawei Wu
@create time: 2019-12-06 23:16
@edit time: 2020-01-15 16:13
@file: /wjwgym-home/vvlab/agents/__init__.py
"""


from .DDPG_base import DDPGBase
from .DQN_base import DQNBase
from .Linear_base import LinearBase