#!/usr/bin/env python
# coding=utf-8
"""
@author: Jiawei Wu
@create time: 2020-04-05 19:45
@edit time: 2020-04-05 20:43
@FilePath: /vvlab/utils/config.py
@desc: config file
"""

import torch
CUDA = torch.cuda.is_available()
